#! /usr/bin/python

## trefnydd_gui.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    trefnydd_gui
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

trefnydd_gui
============

"""


import os, sys
import wx
import wx.html

# in case not installed in python_path
trefnydd_root = os.path.dirname(os.getcwd())
if trefnydd_root not in sys.path:
    sys.path.append(trefnydd_root)

from trefnydd.project import TrefnyddProject
from trefnydd.gui import gui, treedisplay
from trefnydd.utils import utils
from trefnydd.lib import corpus

# gui:
# cribbed from wxPython in Action SplitterExampleFrame
#          and PyCrust/crust.py

class TrefnyddFrame(wx.Frame):
    def __init__(self, parent, projName='', debug=False):
        self.debug = debug
        wx.Frame.__init__(self, parent, -1, 'Trefnydd')
        self.MakeMenuBar()
        self.sp = wx.SplitterWindow(self)
        self.project = None
        #self.InitProject(projName)
        self.selection = None
        self.MakeNotebook()
        self.display = gui.DataDisplay(self.sp, self.debug) 
        self.sp.SplitVertically(self.notebook, self.display)

    def MakeMenuBar(self):
        mbar = wx.MenuBar()

        # File menu
        menu = wx.Menu()

        newSm = wx.Menu()
        item = newSm.Append(-1, "Project")
        self.Bind(wx.EVT_MENU, self.NewProject, item)
        item = newSm.Append(-1, "Corpus")
        self.Bind(wx.EVT_MENU, self.NewCorpus, item)
        menu.AppendMenu(-1, "New ...", newSm)

        openSm = wx.Menu()
        item = openSm.Append(-1, "Project")
        self.Bind(wx.EVT_MENU, self.OpenProject, item)
        menu.AppendMenu(-1, "Open ...", openSm)
        
        item = menu.Append(-1, "Save Project\tCtrl-S")
        self.Bind(wx.EVT_MENU, self.OnSave, item)
        
        item = menu.Append(-1, "Close Project\tCtrl-C")
        self.Bind(wx.EVT_MENU, self.OnClose, item)

        item = menu.Append(wx.ID_EXIT, "Quit\tCtrl-Q")
        self.Bind(wx.EVT_MENU, self.OnExit, item)
        mbar.Append(menu, "File")
        
        # Edit menu
        menu = wx.Menu()
        mbar.Append(menu, "Edit")

        # View menu
        menu = wx.Menu()
        mbar.Append(menu, "View")

        # Help menu
        menu = wx.Menu()
        item = menu.Append(-1, "About")
        self.Bind(wx.EVT_MENU, self.OnAbout, item)
        mbar.Append(menu, "Help")
        
        self.SetMenuBar(mbar)

    def MakeNotebook(self):
        self.notebook = wx.Notebook(self.sp, style=wx.SUNKEN_BORDER)
        self.MakeCorporaTab()
        self.notebook.AddPage(page=self.corporaTab,
                              text='Corpora', select=True)
        self.MakeModelsTab()
        self.notebook.AddPage(page=self.modelsTab, text='Models')
        self.MakeEnginesTab()
        self.notebook.AddPage(page=self.enginesTab, text='Engines')

    def MakeCorporaTab(self):
        self.corporaTab = wx.Notebook(self.notebook, style=wx.SUNKEN_BORDER)
        self.speechTab = treedisplay.CorporaDisplay(self.corporaTab, self.debug)
        self.speechTab.setParent(self)
        if self.project:
            for c in self.project.corpora:
                if isinstance(c, corpus.SpeechCorpus):
                    self.speechTab.addCorpus(c)
        self.corporaTab.AddPage(page=self.speechTab,
                                text='Speech', select=True)
        self.selection = self.speechTab.GetId()
        self.textTab = self.MakeNotebookTab(self.corporaTab, "lightgrey")
        self.corporaTab.AddPage(page=self.textTab, text='Text')
        self.PDTab = treedisplay.PD_Display(self.corporaTab, self.debug)
        if self.project:
            self.PDTab.setParent(self)
        self.corporaTab.AddPage(page=self.PDTab, text='PD')

    def MakeModelsTab(self):
        self.modelsTab = wx.Notebook(self.notebook, style=wx.SUNKEN_BORDER)

        self.AMTab = self.MakeNotebookTab(self.modelsTab, "lightgrey")
        self.modelsTab.AddPage(page=self.AMTab,
                                text='Acoustic Model', select=True)
        self.LMTab = self.MakeNotebookTab(self.modelsTab, "lightgrey")
        self.modelsTab.AddPage(page=self.LMTab, text='Language Model')
        
    def MakeEnginesTab(self):
        self.enginesTab = wx.Notebook(self.notebook, style=wx.SUNKEN_BORDER)

        self.MRCPTab = self.MakeNotebookTab(self.enginesTab, "lightgrey")
        self.enginesTab.AddPage(page=self.MRCPTab,
                                text='MRCP Server', select=True)
        # Any other kinds of engine?

    def MakeNotebookTab(self, parent, bgrndColour):
        tab = wx.Panel(parent, style=wx.SUNKEN_BORDER)
        tab.SetBackgroundColour(bgrndColour)
        return tab

    def OnExit(self, evt):
        # todo: show warning 'does not save!'
        self.Close()

    def OnSave(self, evt):
        if self.project:
            print self.debug, self.project.debug
            print 'TrefnyddFrame.OnSave(): self.project.save()'
            self.project.save()

    def OnAbout(self, evt):
        ab = gui.AboutBox(self)
        ab.ShowModal()
        ab.Destroy()

    def NewProject(self, evt):
        np = wx.FileDialog(self,
                           "Choose Project Name & Home Directory",
                           os.getcwd(), "TrefnyddProject", "", wx.SAVE)
        if np.ShowModal() == wx.ID_OK:
            self.SetProject(np.GetPath())
        np.Destroy()

    def NewCorpus(self, evt):
        # name, location for corpus (default location in project)
        # type: text or speech corpus
        # files to import?
        nc = gui.NewCorpusDialog(self)
        if nc.ShowModal() == wx.ID_OK:
            # TODO: works but inelegant
            corpusName = nc.name.GetValue()
            corpusType = nc.type
            #print corpusName, corpusType
        nc.Destroy()
        if corpusType == 'Speech':
            c = corpus.SpeechCorpus(self.debug)
            c.name = corpusName
            self.speechTab.addCorpus(c)
        elif corpusType == 'Text':
            # todo:
            c = corpus.TextCorpus(self.debug)
            c.name = corpusName
            # self.textTab.addCorpus(c)
        self.project.addCorpus(c)
        
        # TODO: which?
        self.speechTab.Update()
        self.speechTab.Refresh()

    def OpenProject(self, evt):
        np = wx.DirDialog(self,
                          "Choose Project",
                          os.getcwd(),
                          style=wx.DD_DEFAULT_STYLE)
        if np.ShowModal() == wx.ID_OK:
            self.SetProject(np.GetPath())
        np.Destroy()

    def OnClose(self, evt):
        # todo: clear all project stuff
        self.SetProject(None)

    def InitProject(self, project_dirn=None):
        self.project = TrefnyddProject(project_dirn, self.debug)
        projName = 'Trefnydd :: %s' % self.project.name
        self.SetTitle(projName)
        # TODO: either refresh or update will work,
        # but the wxPython docs are no good on which is appropriate
        self.Update()
        # self.Refresh()

    def SetProject(self, project_dirn=None):
        if project_dirn:
            self.project = TrefnyddProject(project_dirn, self.debug)
            projName = 'Trefnydd :: %s' % self.project.name
##             # todo: import project stuff into gui
            for corp in self.project.corpora:
                self.speechTab.addCorpus(corp)
##            # TODO: same for PDs
##             if self.project.pd:
##                 self.PDTab.addPD(self.project.pd)
##                 self.PDTab.setParent(self)
##            for m in self.project.models:
##                self.modelTab.addModel(m)
##            for e in self.project.engines:
##                self.engineTab.addEngine(e)
        else: 
            projName = 'Trefnydd'
            self.project = None
            self.speechTab.clear()
##            self.modelTab.clear()
##            self.engineTab.clear()
        self.SetTitle(projName)
        # TODO: either refresh or update will work,
        # but the wxPython docs are no good on which is appropriate
        self.Update()
        # self.Refresh()
        
    def setPD(self, pd):
        self.project.setPD(pd)
    
    def Display(self, item, parentName):
        self.display.display(item, parentName)

    def getWlistFromCorpora(self):
        wlist = []
        for corpus in self.speechTab.corpora:
            corpus.makeIndex('Word') # todo: danger magic text!
            wlist.extend(corpus.indices['Word'])
##         for corpus in self.textTab.corpora:
##             corpus.makeIndex('Word')
##             wlist.extend(corpus.indices['Word'])
        wlist = utils.uniq(wlist)
        return wlist
    

def main(projName='', debug=False):
    app = wx.PySimpleApp(redirect=True)
    frm = TrefnyddFrame(None, projName, debug)
    frm.SetSize((800,600))
    frm.Show()
    app.MainLoop()


if __name__ == '__main__':
    projName =''
    main(projName, True)
 
